@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 
 ####
 # This script converts an Apple OS X Lion plist file
@@ -10,6 +10,7 @@
 # modification, are permitted. (This is a heavily cut-down "BSD licence".)
 ####
 
+use warnings;
 use strict;
 use Data::Plist;
 use Data::Plist::BinaryReader;
